﻿Namespace Scope
    Module Scope

        Private m_SampleFreq As Int32 = 176400 ' 44100 or 176400 or 192000 or 198450
        Private m_Channels As Int32 = 1
        Private m_NumBuffers As Int32 = 4
        Private m_LenBuffers As Int32 = 4096

        Private m_fifo As FIFO = New FIFO(CInt(m_SampleFreq * 2))
        Private m_Recorder As AudioIn

        Private m_mvDiv As Single = 500
        Private m_msDiv As Single = 100

        Private textFont As Font = New Font("Tahoma", 8)
        Private BackColor As Color = Color.FromArgb(30, 90, 120)
        Private scalePen As Pen = New Pen(Color.FromArgb(45, 58, 70))
        Private tracePen As Pen = New Pen(Color.FromArgb(240, 255, 255))

        Private Sub DataArrived(ByRef data() As Int16)
            m_fifo.AddArray(data)
        End Sub

        Friend Sub Initialize(ByVal pbox As PictureBox, _
                              Optional ByVal InputDeviceIndex As Int32 = -1)
            CloseAll()
            m_Recorder = New AudioIn()
            m_Recorder.InputOn(InputDeviceIndex, _
                               m_SampleFreq, _
                               m_Channels, _
                               m_LenBuffers, _
                               m_NumBuffers, _
                               New AudioIn.BufferDoneEventHandler(AddressOf DataArrived))
        End Sub

        Friend Sub CloseAll()
            If m_Recorder IsNot Nothing Then
                m_Recorder.InputOff()
            End If
        End Sub

        Friend Sub SetVoltage(ByVal mvDiv As Single)
            m_mvDiv = mvDiv
        End Sub

        Friend Sub SetTime(ByVal msDiv As Single)
            m_msDiv = msDiv
        End Sub

        Friend Sub Update(ByVal pbox As PictureBox, _
                          ByVal neg As Boolean)

            'Dim sw1 As Diagnostics.Stopwatch = New Diagnostics.Stopwatch : sw1.Start()
            ' ----------------------------------------------------------------- 
            InitPictureboxImage(pbox)
            ' ----------------------------------------------------------------- 
            Dim i As Int32
            Dim x As Single
            Dim y As Single
            Dim oldx As Int32
            Dim oldy As Single
            Dim h As Int32 = pbox.Image.Height
            Dim w As Int32 = pbox.Image.Width
            Dim h2 As Single = h / 2.0F
            Dim kmv As Single = h * 1000.0F / 8.0F / m_mvDiv
            Dim kw As Single = m_SampleFreq / 100.0F * m_msDiv / w
            Dim preTrigSamples As Int32
            Dim FifoReadIndex As Int32
            ' ----------------------------------------------------------------- 
            Dim g As Graphics = Graphics.FromImage(pbox.Image)
            g.Clear(BackColor)
            ' ----------------------------------------------------------------- draw scale
            For i = 0 To 9
                x = w * i \ 10
                g.DrawLine(scalePen, x, 0, x, h)
            Next
            For i = 0 To 7
                y = h * i \ 8
                g.DrawLine(scalePen, 0, y, w, y)
            Next
            ' ----------------------------------------------------------------- prepare the FIFO read position
            preTrigSamples = Math.Max(CInt(kw * w * 10), m_SampleFreq \ 4)
            FifoReadIndex = CInt(m_fifo.GetWriteIndex - preTrigSamples)
            ' ----------------------------------------------------------------- adaptive trigger
            Static TrigMin As Single
            Static TrigMax As Single
            Dim min As Single = 1
            Dim max As Single = -1
            Dim trigpos As Int32 = 0
            Dim v As Single
            Dim TrigState As Int32 = 0
            For ix As Int32 = 0 To preTrigSamples \ 2
                v = m_fifo.GetElement(FifoReadIndex + ix) / 32768.0F
                If neg Then v = -v
                If TrigState = 0 AndAlso v > TrigMax Then TrigState = 1
                If TrigState = 1 AndAlso v < TrigMin Then TrigState = 2
                If TrigState = 2 AndAlso v > TrigMax / 10 Then
                    TrigState = -1
                    trigpos = ix - CInt(w * kw * 0.11F)
                End If
                If v > max Then max = v
                If v < min Then min = v
            Next
            TrigMin = min * 0.9F
            TrigMax = max * 0.9F
            ' ----------------------------------------------------------------- wave
            g.SmoothingMode = Drawing2D.SmoothingMode.HighQuality
            oldx = -1
            oldy = 0
            For ix As Int32 = 0 To w - 1
                v = m_fifo.GetElement(FifoReadIndex + CInt(ix * kw + trigpos)) / 32768.0F
                If neg Then v = -v
                y = h2 - kmv * v
                If oldx >= 0 Then
                    g.DrawLine(tracePen, oldx, oldy, ix, y)
                    'g.DrawLine(tracePen, ix + 2, y, ix + 3, y)
                End If
                oldx = ix
                oldy = y
            Next
            ' ----------------------------------------------------------------- text
            g.DrawString(m_mvDiv.ToString() & " mV", textFont, Brushes.Azure, 5, h - 20)
            If m_msDiv >= 1 Then
                g.DrawString(m_msDiv.ToString() & " mS", textFont, Brushes.Azure, w - 46, h - 20)
            Else
                g.DrawString((m_msDiv * 1000).ToString() & " uS", textFont, Brushes.Azure, w - 45, h - 20)
            End If
            ' ----------------------------------------------------------------- show
            pbox.Invalidate()
            'ReportString = "Scope time: " & (sw1.Elapsed.TotalMilliseconds * 1000).ToString("0") & " uS"
        End Sub

    End Module
End Namespace
